#include "MainWindow.h"
#include "NWindowsVersion.h"
#include "Globals.h"
#include "NUtilFtns.h"
#include "NTimer.h"
#include "MainLoop.h"

NVec2 GetClickPos(LPARAM lParam)
{
	NVec2 RetVal;

	RetVal.x = float(GET_X_LPARAM(lParam) / Globals::GAME_SCALE);
	RetVal.y = float(GET_Y_LPARAM(lParam) / Globals::GAME_SCALE);

	return RetVal;
}

void SetDragging(NVec2 ClickPos)
{
	Globals::Dragging = NULL;
	Globals::DragOffset = NVec2();

	int ErasePos = -1;

	// Iterate top to bottom and look for first match, then move it to the top.
	for (int i = (int)Globals::Objects[3].size() - 1; i >= 0 && !Globals::Dragging; --i)
	{
		Thing* ThisObj = Globals::Objects[3][i];

		if (ThisObj->bDraggable)
		{
			NVec2 ThisPos = ThisObj->GetPos();
			if (ClickPos.x >= ThisPos.x - ThisObj->Extents.x &&
				ClickPos.x <= ThisPos.x + ThisObj->Extents.x &&
				ClickPos.y >= ThisPos.y - ThisObj->Extents.y &&
				ClickPos.y <= ThisPos.y + ThisObj->Extents.y)
			{
				Globals::Dragging = ThisObj;
				ErasePos = i;

				ThisObj->Stabilize();

				Globals::DragOffset = Globals::Dragging->Pos - ClickPos;
			}
		}
	}

	if (Globals::Dragging &&
		ErasePos >= 0 &&
		ErasePos < (int)Globals::Objects[3].size() - 1)
	{
		Globals::Objects[3].erase(Globals::Objects[3].begin() + ErasePos);
		Globals::Objects[3].push_back(Globals::Dragging);
	}
}

void ReleaseDragging()
{
	// TODO: See whether we're hovering over a target that we can drop this thing on.

	if (Globals::Dragging)
	{
		Globals::Dragging->Destabilize();

		if (Globals::Dragging->Pos.x >= ((Globals::GAME_WIDTH * 0.875) - 48.0f) &&
			Globals::Dragging->Pos.x <= ((Globals::GAME_WIDTH * 0.875) + 48.0f) &&
			Globals::Dragging->Pos.y >= 0 &&		// whatever
			Globals::Dragging->Pos.y <= 56 + 4)			// sure
		{
			// Send it into the sorter!
			int ErasePos = -1;
			for (int i = 0; i < (int)Globals::Objects[3].size() && ErasePos == -1; ++i)
			{
				if (Globals::Objects[3][i] == Globals::Dragging)
				{
					ErasePos = i;
				}
			}
			assert(ErasePos != -1);

			Globals::Objects[3].erase(Globals::Objects[3].begin() + ErasePos);
			Globals::Objects[1].push_back(Globals::Dragging);

			Globals::Sound_MailIn.Play();
		}
		else if (Globals::Dragging->Pos.x >= 112 - 2 &&
			Globals::Dragging->Pos.x <= 128 + 2 &&
			Globals::Dragging->Pos.y >= 104 - 2 &&
			Globals::Dragging->Pos.y <= 120 + 2)
		{
			// Send it into the black hole!

			Mail* ThisMail = (Mail*)(Globals::Dragging);
			if (ThisMail->bUndeliverable)
			{
				// Score points!!
				Globals::DELIVERY_SUCCESSES++;
				Globals::Sound_Vortex.Play();
			}
			else
			{
				// Lose points??
				Globals::DELIVERY_FAILURES++;
				Globals::Sound_Strike.Play();
			}

			int ErasePos = -1;
			for (int i = 0; i < (int)Globals::Objects[3].size() && ErasePos == -1; ++i)
			{
				if (Globals::Objects[3][i] == Globals::Dragging)
				{
					ErasePos = i;
				}
			}
			assert(ErasePos != -1);

			Globals::Objects[3].erase(Globals::Objects[3].begin() + ErasePos);
			SafeDelete(Globals::Dragging);					// TODO: Save this up for a gag instead.
		}
		else
		{
			// Drop it off at a dome!

			int DroppedIndex = -1;

			// The first five things in Objects[0] are always the domes.
			for (int i = 0; i < 5 && DroppedIndex == -1; ++i)
			{
				Thing* ThisTarget = Globals::Objects[0][i];
				if (Globals::Dragging->Pos.x >= ThisTarget->GetPos().x - ThisTarget->Extents.x &&
					Globals::Dragging->Pos.x <= ThisTarget->GetPos().x + ThisTarget->Extents.x &&
					Globals::Dragging->Pos.y >= ThisTarget->GetPos().y - ThisTarget->Extents.y &&
					Globals::Dragging->Pos.y <= ThisTarget->GetPos().y + ThisTarget->Extents.y)
				{
					DroppedIndex = i;
				}
			}

			if (DroppedIndex != -1)
			{
				Dome* Delivery = (Dome*)(Globals::Objects[0][DroppedIndex]);
				Mail* ThisMail = (Mail*)(Globals::Dragging);

				float AbsTime = gTimer->getAbsoluteTime();
				float DeliveryTime = (AbsTime - ThisMail->OriginTime);

				//static std::vector<float> DeliveryTimes;
				//DeliveryTimes.push_back(DeliveryTime);

				if (Delivery->Address == ThisMail->Address)
				{
					// Score points!!
					Globals::DELIVERY_SUCCESSES++;

					Globals::DELIVERY_TOTAL_COUNT++;
					Globals::DELIVERY_TOTAL_TIME += DeliveryTime;

					if (DeliveryTime < Globals::DELIVERY_TIME_LIMIT)
					{
						Globals::DELIVERY_SATISFIED++;
						Globals::Sound_Success.Play();
					}
					else
					{
						Globals::DELIVERY_UNSATISFIED++;
						Globals::Sound_TooSlow.Play();
					}
				}
				else
				{
					// Lose points??
					Globals::DELIVERY_FAILURES++;
					Globals::Sound_Strike.Play();
				}

				int ErasePos = -1;
				for (int i = 0; i < (int)Globals::Objects[3].size() && ErasePos == -1; ++i)
				{
					if (Globals::Objects[3][i] == Globals::Dragging)
					{
						ErasePos = i;
					}
				}
				assert(ErasePos != -1);

				Globals::Objects[3].erase(Globals::Objects[3].begin() + ErasePos);
				SafeDelete(Globals::Dragging);
			}
		}
	}

	Globals::Dragging = NULL;
}

void SetDraggingPos(NVec2 InPos)
{
	if (Globals::Dragging)
	{
		Globals::Dragging->Pos = InPos + Globals::DragOffset;

		Globals::Dragging->Pos.x = Clamp(Globals::Dragging->Pos.x, 0.0f + Globals::Dragging->Extents.x, (float)Globals::GAME_WIDTH - Globals::Dragging->Extents.x);
		Globals::Dragging->Pos.y = Clamp(Globals::Dragging->Pos.y, 0.0f + Globals::Dragging->Extents.y, (float)Globals::GAME_HEIGHT - Globals::Dragging->Extents.y);
	}
}

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_CLOSE:
		PostQuitMessage(0);
		break;

	case WM_LBUTTONDOWN:

		if (Globals::GAME_OVER)
		{
			ResetGame();
		}
		else if (!Globals::GAME_STARTED)
		{
			ResetGame();
			StartGame();
		}
		else
		{
			SetDragging(GetClickPos(lParam));
			SetDraggingPos(GetClickPos(lParam));

			TRACKMOUSEEVENT TrackEvent;
			TrackEvent.cbSize = sizeof(TRACKMOUSEEVENT);
			TrackEvent.dwFlags = TME_LEAVE;
			TrackEvent.dwHoverTime = 0;
			TrackEvent.hwndTrack = hWnd;
			TrackMouseEvent(&TrackEvent);
		}

		break;

	case WM_LBUTTONUP:
	case WM_MOUSELEAVE:
		ReleaseDragging();
		break;

	case WM_MOUSEMOVE:
		SetDraggingPos(GetClickPos(lParam));
		break;

	default:
		break;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

HWND BuildWindow(int Width, int Height, const char* Title, HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR cmdLine, int cmdShow)
{
	WNDCLASSEX wcex;

	ZeroMemory(&wcex, sizeof(wcex));
	wcex.cbSize = sizeof(WNDCLASSEX);

	if (!GetClassInfoEx(hInst, "Main Window", &wcex))
	{
		wcex.cbSize = sizeof(WNDCLASSEX);

		wcex.style			= CS_HREDRAW | CS_VREDRAW;
		wcex.lpfnWndProc	= MainWndProc;
		wcex.cbClsExtra		= 0;
		wcex.cbWndExtra		= 0;
		wcex.hInstance		= hInst;
		wcex.hIcon			= LoadIcon(hInst, NULL);
		wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
		wcex.hbrBackground	= GetSysColorBrush(COLOR_WINDOW);	// COLOR_WINDOW is white.  COLOR_3DFACE will make the window gray, if I want that.
		wcex.lpszMenuName	= NULL;
		wcex.lpszClassName	= "Main Window";
		wcex.hIconSm		= LoadIcon(wcex.hInstance, NULL);

		RegisterClassEx(&wcex);
	}

	DWORD WinFlags = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX;		//WS_OVERLAPPEDWINDOW

	RECT WinRect;
	WinRect.left = 64;
	WinRect.top = 64;
	WinRect.right = WinRect.left + Width;
	WinRect.bottom = WinRect.top + Height;
	AdjustWindowRect(&WinRect, WinFlags, false);

	HWND hWnd = CreateWindowEx(0,		// extra window attributes
		"Main Window",						// window class name
		Title,				// title bar
		WinFlags,		// window style
		WinRect.left,							// x
		WinRect.top,							// y
		WinRect.right - WinRect.left,			// width
		WinRect.bottom - WinRect.top,			// height
		NULL,							// handle to parent window
		NULL,					// handle to menu					// 11/10/2009: OR a control identifier for buttons, etc.!!
		hInst,						// handle to window instance
		NULL);									// extra info

	ShowWindow(hWnd, cmdShow);



	return hWnd;
}
