#pragma once

#include "NMathDefs.h"
#include "NMathFtns.h"

class NColor8;
class NColor32f;

class NColor8
{
public:
	NColor8() : a(255), r(0), g(0), b(0) {}
	NColor8(uchar _rgb) : a(255), r(_rgb), g(_rgb), b(_rgb) {}
	NColor8(uchar _r, uchar _g, uchar _b, uchar _a) : a(_a), r(_r), g(_g), b(_b) {}
	NColor8(uchar _r, uchar _g, uchar _b) : a(255), r(_r), g(_g), b(_b) {}
	explicit NColor8(const NColor32f& c);

	NColor8 operator +(uchar s) const			{return NColor8((uchar)Min(255, r + s), (uchar)Min(255, g + s), (uchar)Min(255, b + s), (uchar)Min(255, a + s));}
	NColor8 operator +(const NColor8& c) const	{return NColor8((uchar)Min(255, r + c.r), (uchar)Min(255, g + c.g), (uchar)Min(255, b + c.b), (uchar)Min(255, a + c.a));}
	NColor8 operator -(uchar s) const			{return NColor8((uchar)Max(0, r - s), (uchar)Max(0, g - s), (uchar)Max(0, b - s), (uchar)Max(0, a - s));}
	NColor8 operator -(const NColor8& c) const	{return NColor8((uchar)Max(0, r - c.r), (uchar)Max(0, g - c.g), (uchar)Max(0, b - c.b), (uchar)Max(0, a - c.a));}
	NColor8 operator *(uchar s) const			{return NColor8((r * s), (g * s), (b * s), (a * s));}
	NColor8 operator *(const NColor8& c) const	{return NColor8((r * c.r)>>8, (g * c.g)>>8, (b * c.b)>>8, (a * c.a)>>8);}		// Meh. Really would want to account for 255 == 1.0 here with float conversions.
	NColor8 operator /(uchar s) const			{return NColor8((r / s), (g / s), (b / s), (a / s));}
	NColor8 operator /(const NColor8& c) const	{return NColor8((r / c.r), (g / c.g), (b / c.b), (a / c.a));}
	void operator +=(uchar s)					{r = (uchar)Min(255, r + s); g = (uchar)Min(255, g + s); b = (uchar)Min(255, b + s); a = (uchar)Min(255, a + s);}
	void operator +=(const NColor8& c)			{r = (uchar)Min(255, r + c.r); g = (uchar)Min(255, g + c.g); b = (uchar)Min(255, b + c.b); a = (uchar)Min(255, a + c.a);}
	void operator -=(uchar s)					{r = (uchar)Max(0, r - s); g = (uchar)Max(0, g - s); b = (uchar)Max(0, b - s); a = (uchar)Max(0, a - s);}
	void operator -=(const NColor8& c)			{r = (uchar)Max(0, r - c.r); g = (uchar)Max(0, g - c.g); b = (uchar)Max(0, b - c.b); a = (uchar)Max(0, a - c.a);}
	void operator *=(uchar s)					{r = (r * s); g = (g * s); b = (b * s); a = (a * s);}
	void operator *=(const NColor8& c)			{r = (r * c.r)>>8; g = (g * c.g)>>8; b = (b * c.b)>>8; a = (a * c.a)>>8;}		// Meh. Really would want to account for 255 == 1.0 here with float conversions.
	void operator /=(uchar s)					{r = (r / s); g = (g / s); b = (b / s); a = (a / s);}
	void operator /=(const NColor8& c)			{r = (r / c.r); g = (g / c.g); b = (b / c.b); a = (a / c.a);}

	NColor8 operator *(float s) const			{return NColor8((uchar)Clamp((int)(float(r) * s), 0, 255),
																(uchar)Clamp((int)(float(g) * s), 0, 255),
																(uchar)Clamp((int)(float(b) * s), 0, 255),
																(uchar)Clamp((int)(float(a) * s), 0, 255));}
	NColor8 operator /(float s) const			{float is = 1.0f / s;
												return NColor8((uchar)Clamp((int)(float(r) * is), 0, 255),
															   (uchar)Clamp((int)(float(g) * is), 0, 255),
															   (uchar)Clamp((int)(float(b) * is), 0, 255),
															   (uchar)Clamp((int)(float(a) * is), 0, 255));}

	NColor8 operator *(const NColor32f& c) const;		// These....aren't implemented anywhere.
	NColor8 operator /(const NColor32f& c) const;

	void operator *=(float s)					{	r = (uchar)Clamp((int)(float(r) * s), 0, 255);
													g = (uchar)Clamp((int)(float(g) * s), 0, 255);
													b = (uchar)Clamp((int)(float(b) * s), 0, 255);
													a = (uchar)Clamp((int)(float(a) * s), 0, 255);}
	void operator /=(float s)					{	float is = 1.0f / s;
													r = (uchar)Clamp((int)(float(r) * is), 0, 255);
													g = (uchar)Clamp((int)(float(g) * is), 0, 255);
													b = (uchar)Clamp((int)(float(b) * is), 0, 255);
													a = (uchar)Clamp((int)(float(a) * is), 0, 255);}

	void operator *=(const NColor32f& c);
	void operator /=(const NColor32f& c);

	bool operator !=(const NColor8& c)			{return (r != c.r) || (g != c.g) || (b != c.b) || (a != c.a);}
	bool operator ==(const NColor8& c)			{return (r == c.r) && (g == c.g) && (b == c.b) && (a == c.a);}

	// I SHOULD just be able to use *(nulong*)this as an RGBA value, but if not, there's this.
	nulong ToARGB() const {return (a<<24)|(r<<16)|(g<<8)|(b);}
	nulong ToRGBA() const {return (r<<24)|(g<<16)|(b<<8)|(a);}
	nulong ToBGRA() const {return (b<<24)|(g<<16)|(r<<8)|(a);}

	void FromARGB(nulong InVal)
	{
		r = uchar((InVal>>16) & 0x000000ff);
		g = uchar((InVal>>8) & 0x000000ff);
		b = uchar((InVal>>0) & 0x000000ff);
		a = uchar((InVal>>24) & 0x000000ff);
	}
	void FromRGBA(nulong InVal)
	{
		r = uchar((InVal>>24) & 0x000000ff);
		g = uchar((InVal>>16) & 0x000000ff);
		b = uchar((InVal>>8) & 0x000000ff);
		a = uchar((InVal>>0) & 0x000000ff);
	}
    void FromBGRA(nulong InVal)
	{
		r = uchar((InVal>>8) & 0x000000ff);
		g = uchar((InVal>>16) & 0x000000ff);
		b = uchar((InVal>>24) & 0x000000ff);
		a = uchar((InVal>>0) & 0x000000ff);
	}

	// No reason these need to be private.
	// They are defined in this order so that I can pass them to D3D without swizzling.
	uchar b,g,r,a;

	static NColor8 FromHex(const char* HexString);
	static byte FromHexHelper_HalfByte(const char*& itr);
	static byte FromHexHelper_Byte(const char*& itr);
};

class NColor32f
{
public:
	NColor32f() : a(1.0f), r(0.0f), g(0.0f), b(0.0f) {}
	NColor32f(float _rgb) : a(1.0f), r(_rgb), g(_rgb), b(_rgb) {}
	NColor32f(float _r, float _g, float _b, float _a) : a(_a), r(_r), g(_g), b(_b) {}
	NColor32f(float _r, float _g, float _b) : a(1.0f), r(_r), g(_g), b(_b) {}
	NColor32f(const NColor8& c);

	NColor32f operator +(float s) const				{return NColor32f(r + s, g + s, b + s, a + s);}
	NColor32f operator +(const NColor32f& c) const	{return NColor32f(r + c.r, g + c.g, b + c.b, a + c.a);}
	NColor32f operator -(float s) const				{return NColor32f(r - s, g - s, b - s, a - s);}
	NColor32f operator -(const NColor32f& c) const	{return NColor32f(r - c.r, g - c.g, b - c.b, a - c.a);}
	NColor32f operator *(float s) const				{return NColor32f(r * s, g * s, b * s, a * s);}
	NColor32f operator *(const NColor32f& c) const	{return NColor32f(r * c.r, g * c.g, b * c.b, a * c.a);}
	NColor32f operator /(float s) const				{float is = 1.0f / s; return NColor32f(r * is, g * is, b * is, a * is);}
	NColor32f operator /(const NColor32f& c) const	{return NColor32f(r / c.r, g / c.g, b / c.b, a / c.a);}
	void operator +=(float s)						{r += s; g += s; b += s; a += s;}
	void operator +=(const NColor32f& c)				{r += c.r; g += c.g; b += c.b; a += c.a;}
	void operator -=(float s)						{r -= s; g -= s; b -= s; a -= s;}
	void operator -=(const NColor32f& c)				{r -= c.r; g -= c.g; b -= c.b; a -= c.a;}
	void operator *=(float s)						{r *= s; g *= s; b *= s; a *= s;}
	void operator *=(const NColor32f& c)				{r *= c.r; g *= c.g; b *= c.b; a *= c.a;}
	void operator /=(float s)						{float is = 1.0f / s; r *= is; g *= is; b *= is; a *= is;}
	void operator /=(const NColor32f& c)				{r /= c.r; g /= c.g; b /= c.b; a /= c.a;}

	float r,g,b,a;		// defined in this order for synchronization with D3D shaders, hopefully
						// (I'd like to be able to do something like:
						// g3d->setShaderFloatArray("lightColor", 4, (float*)&aNColor32fObject);

	// TODO: NColor32f *, *=, /, and /= uchar
	// TODO: NColor32f *, *=, /, and /= NColor8
	// (Just haven't implemented 'em 'cause I don't see ever using 'em, but who knows...)

	//bool operator ==(const NColor32f& c)			{return (r == c.r) && (g == c.g) && (b == c.rb && (a == c.a);}
};
