#include "NFileIO.h"
#include "NText.h"
#include "NUtilFtns.h"
#include <errno.h>

#include <shlobj.h>
#include <shlwapi.h>

int GetFileSize(FILE* file)
{
	int error;

	error = fseek(file, 0, SEEK_END);
	assert(error == 0);

	int length = ftell(file);

	error = fseek(file, 0, SEEK_SET);
	assert(error == 0);

	return length;
}

// 07/31/2011: Appending a zero to the end of this is dumb.
// It assumes this is a text file. But whatever.
int ReadFileToBuffer(FILE* file, char*& buffer)
{
	size_t error;
	int length = GetFileSize(file);

	buffer = new char[length+1];

	error = fread_s(buffer, sizeof(char) * length, sizeof(char), length, file);
	assert((int)error == length);

	buffer[length] = 0;

	return length;
}

bool FileExists(const char* filename)
{
	bool RetVal = false;

	if (filename && strlen(filename) > 0)
	{
		FILE* file = NULL;
		errno_t error = fopen_s(&file, filename, "rb");

		if (error == 0 && file != 0)
		{
			fclose(file);
			RetVal = true;
		}
		else if (error == EACCES)
		{
#ifdef _DEBUG
			bool FILE_EXISTS_BUT_LOCKED = false;
			opt_assert(FILE_EXISTS_BUT_LOCKED);
			RetVal = true;
#endif
		}
	}

	return RetVal;
}

// Will return false if the file does not exist
bool GetChecksum(const char* filename, uint& OutSum)
{
	bool RetVal = false;
	OutSum = 0;

	if (FileExists(filename))
	{
		RetVal = true;

		FILE* file;
		errno_t error = fopen_s(&file, filename, "rb");

		if (error == 0 && GetFileSize(file) > 0)
		{
			while (!feof(file))
			{
				uint word = 0;
				fread_s(&word, sizeof(word), sizeof(word), 1, file);
				OutSum = OutSum ^ word;
			}

			fclose(file);
		}
	}

	return RetVal;
}

void ListAllFiles(const char* basepath, const char* filter, std::vector<NString>& OutFiles, bool bIncludeSubfolders, const char* subfolderpath)
{
	WIN32_FIND_DATA FileData;

	NString Appended = NString(basepath) + NString(subfolderpath) + NString(filter);

	HANDLE hFind = FindFirstFile(*Appended, &FileData);

	if (hFind != INVALID_HANDLE_VALUE)
	{
		do
		{
			if (!(FileData.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN))
			{
				if (FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					if (bIncludeSubfolders &&
						_stricmp(FileData.cFileName, ".") != 0 &&
						_stricmp(FileData.cFileName, "..") != 0)
					{
						NString Subfolder = NString(subfolderpath) + NString(FileData.cFileName) + NString(FOLDER_SEPARATOR);
						ListAllFiles(basepath, filter, OutFiles, bIncludeSubfolders, *Subfolder);
					}
				}
				else
				{
					OutFiles.push_back(NString(subfolderpath) + NString(FileData.cFileName));
				}
			}
		}
		while (FindNextFile(hFind, &FileData));

		FindClose(hFind);
	}
}

// Generalizing this pattern since I use it so often.
// Could be a macro but whatever.
bool IsVersion(int InMajor, int InMinor, int TestMajor, int TestMinor)
{
	return (
		(InMajor > TestMajor) ||
		((InMajor == TestMajor) && (InMinor >= TestMinor)));
}
