#pragma once

#include <stdio.h>
#include "NWindowsVersion.h"
#include <assert.h>
#include "NMathDefs.h"
#include "NColor.h"

// NERD: Add REAL 8-bit grayscale image support, plus support for palettized 8-bit images.
// NERD: Allow 16-bit bitmap support.  A lot of these functions expect strictly 1, 3, or 4 channels.
// (See Gaia code for how to correctly save both of these file types.)

class NImage
{
public:
	NImage() {ZeroMemory(this, sizeof(NImage));}
	NImage(const NImage& Other);
	void operator =(const NImage& Other);
	~NImage() {free();}

	bool loadBMP(const char* filename);
	bool saveBMP(const char* filename);

	void loadPAL(const char* filename);

	bool init(int w, int h, int chan, int bpp=-1);

	HDC getHDC() const {return hdc;}
	int getWidth() const {return width;}
	int getHeight() const {return height;}
	int getChannels() const {return channels;}
	int getStride() const {return stride;}

	NColor8 maskColor;

	NColor8 getPixel(int x, int y) const;
	void setPixel(int x, int y, NColor8 color);
	void setPixel(int x, int y, uchar value);

	// TODO: Make these safe in case there is no palette.
	RGBQUAD getPaletteEntry(int i) const {return palette[i];}
	void setPaletteEntry(int i, NColor8 color) {palette[i].rgbRed = color.r;palette[i].rgbGreen = color.g;palette[i].rgbBlue = color.b;}

	// This is dangerous, but necessary in order to keep my old code intact.
	// Look into refactoring.
	// Idea: Create a friend class ("Blitter" or whatnot) that has access to private pixel info.
	unsigned char* getPixels() const {return pixels;}

	friend class NBlitter;
	friend class NTexture;

	void clear();
	void free();

private:

	BITMAPFILEHEADER fileheader;
	BITMAPINFOHEADER infoheader;

	HBITMAP bmp;
	HBITMAP oldbmp;

	int width;
	int height;
	int channels;
	int stride;

	int NumColorsInPalette;

	RGBQUAD* palette;
	unsigned char* pixels;

	HDC hdc;
};

// This class provides static functions for blitting images.
// This is done to protect against getting a pointer to an image's pixel array and messing around with it.
class NBlitter
{
public:
	static bool Blit(NImage& dest, const NImage& src, int dest_x, int dest_y, int src_x, int src_y, int w, int h, bool doScissorChecks);
	static bool MaskedBlit(NImage& dest, const NImage& src, int dest_x, int dest_y, int src_x, int src_y, int w, int h, bool doScissorChecks);
	static bool TransparentBlit(NImage& dest, const NImage& src, int dest_x, int dest_y, int src_x, int src_y, int w, int h, bool doScissorChecks, float opacity);
	static bool AlphaBlit(NImage& dest, const NImage& src, int dest_x, int dest_y, int src_x, int src_y, int w, int h, bool doScissorChecks);
	static bool ScaleUpBlit(NImage& dest, const NImage& src, int dest_x, int dest_y, int src_x, int src_y, int dest_w, int dest_h, int src_w, int src_h, bool doScissorChecks, bool filter);
	static bool ScaleDownBlit(NImage& dest, const NImage& src, int dest_x, int dest_y, int src_x, int src_y, int dest_w, int dest_h, int src_w, int src_h, bool doScissorChecks, bool filter);
};
