#include "Thing.h"
#include "NMathFtns.h"
#include "NTimer.h"
#include "Globals.h"

void Mail::UpdateWander(float AbsTime, float DeltaTime)
{
	WanderMag = Smooth(WanderMag, TargetWanderMag, 2.0f, DeltaTime);

	WanderOffset.x = fastcos(AbsTime + WanderPhase);
	WanderOffset.y = fastsin(AbsTime*2.0f + WanderPhase) * 0.5f;
}

void Mail::UpdateTarget(float AbsTime, float DeltaTime)
{
	if (bTargeting)
	{
		Pos = Smooth(Pos, TargetPos, 1.0f, DeltaTime);
	}
}

void Mail::Stabilize()
{
	Pos = GetPos();
	WanderMag = 0.0f;
	TargetWanderMag = 0.0f;
	bTargeting = false;
}

void Mail::Destabilize()
{
	//WanderMag = 8.0f;
	TargetWanderMag = 8.0f;
	UpdateWander(gTimer->getAbsoluteTime(), gTimer->getDeltaTime());
	Pos -= (WanderOffset * WanderMag);
}

bool Mail::HasValidAddress()
{
	bool bRetVal = true;
	bUndeliverable = false;

	if (Address == OriginAddress)
	{
		bRetVal = false;
	}

	if (bRetVal)
	{
		bool bFound = false;

		// The first five things in Objects[0] are always the domes.
		for (int i = 0; i < 5 && !bFound; ++i)
		{
			Dome* Origin = (Dome*)(Globals::Objects[0][i]);
			if (Address == Origin->Address)
			{
				bFound = true;
			}
		}

		if (!bFound)
		{
			if (Globals::GRACE_ADDRESSES > 0)
			{
				bRetVal = false;
			}
		}

		bUndeliverable = !bFound;
	}

	return bRetVal;
}

void Mail::AssignAddress()
{
	if (Address < 0)
	{
		do
		{
			Address = random_fl(8, 0);
		}
		while (!HasValidAddress());

		if (Globals::GRACE_ADDRESSES > 0)
		{
			Globals::GRACE_ADDRESSES--;
		}

		BitBlt(Sprite.Sprite.getHDC(), 0, 0, Sprite.Sprite.getWidth(), Sprite.Sprite.getHeight(), Globals::MailOut.getHDC(), 0, 0, SRCCOPY);
		Globals::Numerals.Blit(Sprite.Sprite.getHDC(), 3, 2, 2 + (Address * 8), 1, 4, 5);
	}
}

std::vector<int> Dome::AvailableAddresses;

void Dome::InitAddresses()
{
	AvailableAddresses.clear();
	for (int i = 0; i < 8; ++i)
	{
		AvailableAddresses.push_back(i);
	}
}

void Dome::AssignAddress()
{
	int Index = random_fl((int)AvailableAddresses.size(), 0);
	Address = AvailableAddresses[Index];
	AvailableAddresses.erase(AvailableAddresses.begin() + Index);

	Globals::Numerals.Blit(Sprite.Sprite.getHDC(), 14, 25, 2 + (Address * 8), 1, 4, 5);
}
