#pragma once

#include "NVec2.h"
#include "MaskedImage.h"
#include "NMathDefs.h"
#include <vector>

class Thing
{
public:
	Thing() : bDraggable(false) {}

	NVec2 Pos;
	NVec2 Extents;

	virtual NVec2 GetPos() {return Pos;}
	virtual void Stabilize() {}
	virtual void Destabilize() {}

	// Sure.
	MaskedImage Sprite;

	bool bDraggable;
};

class Dome : public Thing
{
public:
	Dome() : Address(-1) {}

	static std::vector<int> AvailableAddresses;
	static void InitAddresses();

	int Address;
	void AssignAddress();
};

class Mail : public Thing
{
public:
	Mail() : OriginAddress(-1), Address(-1), bTargeting(false), WanderMag(8.0f), TargetWanderMag(8.0f), OriginTime(-1.0f), bUndeliverable(false) {WanderPhase = random_fl() * TAU;}

	NVec2 WanderOffset;
	float WanderPhase;
	float WanderMag;
	float TargetWanderMag;

	virtual NVec2 GetPos() {return Pos + (WanderOffset * WanderMag);}
	virtual void Stabilize();
	virtual void Destabilize();

	bool bTargeting;
	NVec2 TargetPos;

	void UpdateWander(float AbsTime, float DeltaTime);
	void UpdateTarget(float AbsTime, float DeltaTime);

	int OriginAddress;
	int Address;
	void AssignAddress();

	bool HasValidAddress();

	float OriginTime;
	bool bUndeliverable;
};
