#include "main.h"

#include "time.h"

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR cmdLine, int cmdShow)
{
#define _WITH_MEM_CHECKS
#ifdef _DEBUG
#ifdef _WITH_MEM_CHECKS
	int flag;
	flag = _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
	//flag |= _CRTDBG_CHECK_ALWAYS_DF;				// This is extremely slow and not terribly useful.
	flag |= _CRTDBG_ALLOC_MEM_DF;
	flag |= _CRTDBG_LEAK_CHECK_DF;
	_CrtSetDbgFlag(flag);
#endif
#endif

	srand(int(time(NULL)));

	Globals::gWnd = BuildWindow(Globals::GAME_WIDTH * Globals::GAME_SCALE, Globals::GAME_HEIGHT * Globals::GAME_SCALE, "Interplanetary Postal Service", hInst, hPrevInst, cmdLine, cmdShow);

	Globals::gDC = GetDC(Globals::gWnd);

	Audio::Init(Globals::gWnd);

	Globals::Sound_Compose.Load("compose.wav");
	Globals::Sound_MailIn.Load("mailin.wav");
	Globals::Sound_MailOut.Load("mailout.wav");
	Globals::Sound_Success.Load("success.wav");
	Globals::Sound_Strike.Load("strike.wav");
	Globals::Sound_TooSlow.Load("tooslow.wav");
	Globals::Sound_Vortex.Load("vortex.wav");

	Globals::Backbuffer1.init(Globals::GAME_WIDTH, Globals::GAME_HEIGHT, 3);
	Globals::Backbuffer2.init(Globals::GAME_WIDTH, Globals::GAME_HEIGHT, 3);
	Globals::Backbuffer = &Globals::Backbuffer1;
	Globals::Backdrop.loadBMP("background.bmp");
	Globals::MailOut.loadBMP("mail_out.bmp");

	Globals::Numerals.Load("numerals.bmp", NColor8(255,0,255));
	Globals::BlackHole.Load("blackhole.bmp", NColor8(255,0,255));

	// Four layers:
	// 1. Background. Domes and sorting back.
	// 2. Middle. Mail moving through sorter.
	// 3. Foreground. Front of sorter.
	// 4. Dynamic objects. Mail outside sorter. (All draggable things.)
	for (int i = 0; i < 4; ++i)
	{
		Globals::Objects.push_back(std::vector<Thing*>());
	}

	for (int i = 0; i < 12; ++i)
	{
		Particle NewInPart;
		Particle NewOutPart;

		NewInPart.Life = random_fl() * Globals::PART_LIFE;
		NewOutPart.Life = random_fl() * Globals::PART_LIFE;

		NewInPart.Pos.x = float(Globals::GAME_WIDTH) * 0.875f;
		NewInPart.Pos.y = float(Globals::GAME_HEIGHT) * 0.125f;

		NewOutPart.Pos.x = float(Globals::GAME_WIDTH) * 0.875f;
		NewOutPart.Pos.y = float(Globals::GAME_HEIGHT) * 0.825f;

		NewInPart.Pos.x += (random_fl() - 0.5f) * 48.0f;
		NewInPart.Pos.y += (random_fl() - 0.5f) * 32.0f;

		NewOutPart.Pos.x += (random_fl() - 0.5f) * 48.0f;
		NewOutPart.Pos.y += (random_fl() - 0.5f) * 32.0f;

		Globals::InParticles.push_back(NewInPart);
		Globals::OutParticles.push_back(NewOutPart);
	}

	SelectObject(Globals::Backbuffer1.getHDC(), (HFONT)GetStockObject(ANSI_VAR_FONT));
	SelectObject(Globals::Backbuffer2.getHDC(), (HFONT)GetStockObject(ANSI_VAR_FONT));

	SetTextColor(Globals::Backbuffer1.getHDC(), RGB(255,255,255));
	SetBkMode(Globals::Backbuffer1.getHDC(), TRANSPARENT);
	SetBkColor(Globals::Backbuffer1.getHDC(), RGB(0,0,0));

	SetTextColor(Globals::Backbuffer2.getHDC(), RGB(255,255,255));
	SetBkMode(Globals::Backbuffer2.getHDC(), TRANSPARENT);
	SetBkColor(Globals::Backbuffer2.getHDC(), RGB(0,0,0));

	ResetGame();

	MainLoop();

	for (int i = 0; i < (int)Globals::Objects.size(); ++i)
	{
		ClearObjects(Globals::Objects[i]);
	}

	ReleaseDC(Globals::gWnd, Globals::gDC);

	Audio::Deinit();
	NStringMgr::DestroyInstance();
	NHashedStringTable::DestroyInstance();
}
